"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const webpack = require("webpack");
const merge = require("webpack-merge");
const babel_rule_converter_1 = require("./babel.rule.converter");
const babel_target_chunk_id_updater_1 = require("./babel.target.chunk.id.updater");
const dependency_util_1 = require("./dependency.util");
const excluded_packages_1 = require("./excluded.packages");
const plugin_name_1 = require("./plugin.name");
/**
 * @internal
 */
class BabelTargetCompilerFactory {
    constructor(compilation, compilationTargets, inputContext, inputs, configTemplate, plugins, exclude) {
        this.compilation = compilation;
        this.compilationTargets = compilationTargets;
        this.inputContext = inputContext;
        this.inputs = inputs;
        this.configTemplate = configTemplate;
        this.plugins = plugins;
        this.exclude = exclude;
    }
    static createConfigResolveAlias(inputs) {
        return inputs
            .filter(tempAsset => tempAsset.name.endsWith('.js'))
            .reduce((result, tempAsset) => {
            result[tempAsset.name.replace(/\.js$/, '')] = tempAsset.path;
            return result;
        }, {});
    }
    static createConfigEntry(alias) {
        return Object.keys(alias);
    }
    static createConfigBase(target, entry, alias, template, context, inputContext, pluginsFn) {
        const plugins = typeof (pluginsFn) === 'function' ?
            pluginsFn(target) : [];
        return merge({}, template, {
            entry,
            resolve: {
                alias,
                modules: [
                    // need to point the child compiler at the right node_modules folder since it would
                    // otherwise be looking in the temp dir where the intermediate files are written
                    path.resolve(context, 'node_modules'),
                    inputContext,
                ],
            },
            module: {
                rules: [
                    {
                        test: /\.js$/,
                        use: [
                            path.resolve(__dirname, 'unnest.sourcemap.loader'),
                            'source-map-loader',
                        ],
                        enforce: 'pre',
                    }
                ],
            },
            context,
            devtool: 'source-map',
            // FIXME: get from configuration!
            mode: 'development',
            plugins: [
                new babel_target_chunk_id_updater_1.BabelTargetChunkIdUpdater(target),
                ...plugins,
            ],
        });
    }
    createConfig(target) {
        const alias = BabelTargetCompilerFactory.createConfigResolveAlias(this.inputs);
        const entry = BabelTargetCompilerFactory.createConfigEntry(alias);
        const config = BabelTargetCompilerFactory.createConfigBase(target, entry, alias, this.configTemplate, this.compilation.compiler.context, this.inputContext, this.plugins);
        const exclude = [
            ...excluded_packages_1.STANDARD_EXCLUDED,
            ...this.exclude,
            ...this.getIgnoredModules(),
        ];
        // reassign the babel loader options
        const babelRules = new babel_rule_converter_1.BabelRuleConverter().convertLoaders(config.module.rules, exclude, target.options);
        if (!babelRules.converted) {
            config.module.rules.push({
                test: /\.js$/,
                exclude,
                use: [
                    {
                        loader: babel_rule_converter_1.BABEL_LOADER,
                        options: target.options,
                    },
                ],
            });
        }
        return config;
    }
    getIgnoredModules() {
        const depUtil = new dependency_util_1.DependencyUtil();
        const deps = depUtil.getDependencies(this.compilation);
        return deps.libs
            .filter((dep) => dep.usesCommonJs)
            .map((dep) => `node_modules/${dep.libName}`);
    }
    createCompiler(target) {
        const config = this.createConfig(target);
        const childCompiler = webpack(config);
        childCompiler.name = `${plugin_name_1.CHILD_COMPILER_PREFIX}${target.key}`;
        this.compilationTargets[childCompiler.name] = target;
        return childCompiler;
    }
}
exports.BabelTargetCompilerFactory = BabelTargetCompilerFactory;
//# sourceMappingURL=babel.target.compiler.factory.js.map